#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_errno.h>

#include "vast_limits.h"

#include "stetson_variability_indexes.h"

//#include "vast_math.h"

#include "get_number_of_measured_images_from_vast_summary_log.h"

//int get_number_of_measured_images_from_vast_summary_log();

#include "detailed_error_messages.h"

void sort_log_file_in_mag(){
 int i,n;
 char **file_content;
 
 double * mag;
 
 FILE *f;
 
 size_t * p; // for index sorting
 
 unsigned int number_of_lines_in_file,longest_line_length,current_line_length;
 
 char buf[DATA_PARSER_C_MAX_STR_LENGTH];

 // Count lines in file
 number_of_lines_in_file=longest_line_length=0;
 f=fopen("vast_lightcurve_statistics.log","r");
 if(f==NULL){fprintf(stderr,"ERROR in create_data.c : cannot open vast_lightcurve_statistics.log for reading!\n");report_lightcurve_statistics_computation_problem();exit(1);}
 while(NULL!=fgets(buf,DATA_PARSER_C_MAX_STR_LENGTH,f)){
  buf[DATA_PARSER_C_MAX_STR_LENGTH-1]='\0'; // just in case
  current_line_length=strlen(buf);
  if( longest_line_length<current_line_length )longest_line_length=current_line_length;
  number_of_lines_in_file++;
 }
 fclose(f); // close file for now, we'll re-open it later
 longest_line_length++; // to account for the final '\n' at the end of the line that is not visible to strlen()
 number_of_lines_in_file++; // Oh, this is the non-obvious one! It is needed for the last fgets() that will return NULL
 fprintf(stderr,"vast_lightcurve_statistics.log: %d %d\n",number_of_lines_in_file,longest_line_length);

 
 mag=(double *)malloc(number_of_lines_in_file*sizeof(double));
 if( NULL==mag ){fprintf(stderr,"memory error\n");exit(1);}
 
 file_content=(char **)malloc(number_of_lines_in_file*sizeof(char *));
 if( NULL==file_content ){fprintf(stderr,"memory error\n");exit(1);}
 
 
 for(i=0;i<number_of_lines_in_file;i++)file_content[i]=(char *)malloc(longest_line_length*sizeof(char));
 
 f=fopen("vast_lightcurve_statistics.log","r");
 if(f==NULL){
  fprintf(stderr,"ERROR in create_data.c : cannot open vast_lightcurve_statistics.log for reading!\n");
  report_lightcurve_statistics_computation_problem();
  free(mag);
  for(i=0;i<number_of_lines_in_file;i++)free(file_content[i]);
  free(file_content);
  exit(1);
 }
 i=0;
 while( NULL!=fgets( file_content[i], longest_line_length, f) ){
  //fprintf(stderr,"--- %d ---\n",i);
  file_content[i][longest_line_length-1]='\0'; // just in case
  sscanf(file_content[i],"%lf %s",&mag[i],buf);
  //fprintf(stderr,"%d   %lf\n#%s#\n",i,mag[i],file_content[i]);
  i++;
 }
 fclose(f);
 n=i;
 
 p=malloc(n*sizeof(size_t));
 if( p==NULL ){fprintf(stderr,"ERROR in create_data.c - cannot allocate memory\n");exit(1);}
 // don't forget to initialize the index array "p"!
 /// WHY initialize the index array "p"????
 ///for(i=0;i<n-1;i++)p[i]=(size_t)i;
 
 gsl_sort_index( p, mag, 1, n); // The elements of p give the index of the array element which would have been stored in that position if the array had been sorted in place.  
                                              // The array data is not changed. 

 f=fopen("vast_lightcurve_statistics.log","w");
 for(i=0;i<n;i++){
  fprintf(f,"%s",file_content[p[i]]);
 }
 fclose(f);
 
 free(p);
 for(i=0;i<number_of_lines_in_file;i++)free(file_content[i]);
 free(file_content);
 free(mag);
 return;
}


/*
void sort_log_file_in_mag(){
 int i,n;
 char **file_content;
 
 double * mag;
 
 FILE *f;
 
 size_t * p; // for index sorting
 
 
 mag=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
 if( NULL==mag ){fprintf(stderr,"memory error\n");exit(1);}
 
 file_content=(char **)malloc(MAX_NUMBER_OF_STARS*sizeof(char *));
 if( NULL==file_content ){fprintf(stderr,"memory error\n");exit(1);}
 for(i=0;i<MAX_NUMBER_OF_STARS;i++)file_content[i]=(char *)malloc(DATA_PARSER_C_MAX_STR_LENGTH*sizeof(char));
 
 f=fopen("vast_lightcurve_statistics.log","r");
 if(f==NULL){
  fprintf(stderr,"ERROR in create_data.c : cannot open vast_lightcurve_statistics.log for reading!\n");
  report_lightcurve_statistics_computation_problem();
  free(mag);
  for(i=0;i<MAX_NUMBER_OF_STARS;i++)free(file_content[i]);
  free(file_content);
  exit(1);
 }
 i=0;
 while( NULL!=fgets( file_content[i], DATA_PARSER_C_MAX_STR_LENGTH, f) ){
  sscanf(file_content[i],"%lf",&mag[i]);
  i++;
 }
 fclose(f);
 n=i;
 
 p=malloc(n*sizeof(size_t));
 if( p==NULL ){fprintf(stderr,"ERROR in create_data.c - cannot allocate memory\n");exit(1);}
 // don't forget to initialize the index array "p"!
 for(i=0;i<n-1;i++)p[i]=(size_t)i;
 
 gsl_sort_index( p, mag, 1, n); // The elements of p give the index of the array element which would have been stored in that position if the array had been sorted in place.  
                                              // The array data is not changed. 

 f=fopen("vast_lightcurve_statistics.log","w");
 for(i=0;i<n;i++){
  fprintf(f,"%s",file_content[p[i]]);
 }
 fclose(f);
 
 for(i=0;i<MAX_NUMBER_OF_STARS;i++)free(file_content[i]);
 free(file_content);
 return;
}

*/

int main(){
 FILE * legacy_data_file;
 FILE * extended_data_file;


 DIR *dp;
 struct dirent *ep;

 FILE *lightcurvefile;
 double jd,mag,magerr,x,y,app;
 char string[FILENAME_LENGTH];
 
 int number_of_measured_images_from_vast_summary_log=get_number_of_measured_images_from_vast_summary_log();
 int n_points_to_drop=MIN((int)(0.1*number_of_measured_images_from_vast_summary_log),STAT_NDROP);
 // Handle the special situation if there is no vast_summary.log file
 if( number_of_measured_images_from_vast_summary_log==MAX_NUMBER_OF_OBSERVATIONS )n_points_to_drop=0;
 
 double x_ref,y_ref;
 double m_median,sigma_series;
 int i,j; // just counters
 double * m;
 double * merr;
 double * w;
 double a,b,d;
 
 double m_mean,modified_sigma_series,MAD_scaled_to_sigma,lag1_autocorrelation;
 
 double skewness,kurtosis;

 double I,J,K,L; // Stetson's variability indexes
 double J_clip,L_clip,J_time,L_time; // Modified Stetson's variability indexes
 
 double I_sign_only;
 
 double * jday;
 
 int points_in_lightcurve;
 
 double RoMS;
 
 double AGN_v;
 
 double N3;
 
 double excursions;

 double eta,E_A,SB;
 
 //unsigned int p; // for index sorting

 a=b=2;

 legacy_data_file=fopen("data","w");
 if(NULL==legacy_data_file){fprintf(stderr,"ERROR in create_data.c - cannot open file data for writing!\n");return 1;}
 extended_data_file=fopen("vast_lightcurve_statistics.log","w");
 if(NULL==extended_data_file){fprintf(stderr,"ERROR in create_data.c - cannot open file vast_lightcurve_statistics.log for writing!\n");return 1;}

 
 m=malloc(number_of_measured_images_from_vast_summary_log*sizeof(double));
 if( m==NULL ){fprintf(stderr,"ERROR in create_data.c - cannot allocate memory\n");return 1;}
 merr=malloc(number_of_measured_images_from_vast_summary_log*sizeof(double));
 if( merr==NULL ){fprintf(stderr,"ERROR in create_data.c - cannot allocate memory\n");return 1;}
 w=malloc(number_of_measured_images_from_vast_summary_log*sizeof(double));
 if( w==NULL ){fprintf(stderr,"ERROR in create_data.c - cannot allocate memory\n");return 1;}
 jday=malloc(number_of_measured_images_from_vast_summary_log*sizeof(double));
 if( jday==NULL ){fprintf(stderr,"ERROR in create_data.c - cannot allocate memory\n");return 1;}
 //p=malloc(number_of_measured_images_from_vast_summary_log*sizeof(unsigned int));
 //if( p==NULL ){fprintf(stderr,"ERROR in create_data.c - cannot allocate memory\n");return 1;}
 
 
 dp = opendir ("./");
 if (dp != NULL){
  //fprintf(stderr,"Working...\nPlease, PLEASE, be patient!!!\n");
  while( ep=readdir(dp) ){
   if( strlen(ep->d_name)<8 )continue; // make sure the filename is not too short for the following tests
   if( ep->d_name[0]=='o' && ep->d_name[1]=='u' && ep->d_name[2]=='t' && ep->d_name[strlen(ep->d_name)-1]=='t' && ep->d_name[strlen(ep->d_name)-2]=='a' && ep->d_name[strlen(ep->d_name)-3]=='d' ){
    //puts (ep->d_name);
    lightcurvefile=fopen(ep->d_name,"r");
    if( NULL==lightcurvefile ){
     fprintf(stderr,"ERROR: Can't open file %s\n",ep->d_name);
     exit(1);
    }
    i=0;
    while(-1<fscanf(lightcurvefile,"%lf %lf %lf %lf %lf %lf %s",&jd,&mag,&magerr,&x,&y,&app,string)){
     if(i==0){x_ref=x;y_ref=y;}
     if( 1==isnormal(mag) && 1==isnormal(magerr) ){
      if( mag!=0.0 && magerr!=0.0 ){
       m[i]=mag;
       merr[i]=magerr;
       jday[i]=jd;
       i++;
       if(i>=MAX_NUMBER_OF_OBSERVATIONS){fprintf(stderr,"ERROR (src/create_data.c): i>=MAX_NUMBER_OF_OBSERVATIONS=%d\n",MAX_NUMBER_OF_OBSERVATIONS);fclose(lightcurvefile);return 1;}
      }
     }
    }
    fclose(lightcurvefile);
    
    // Skip the star is it has insufficient number of measurements
    ///
    /// WHAT, really??????????????????????????????????
    ///
    ///
    //if( i<MIN(TESTOR_MIN_NUMBER_OF_POINTS,(int)(0.5*number_of_measured_images_from_vast_summary_log)) )continue;
    //fprintf(stderr,"%s  i=%d\n",ep->d_name,i);
    
    // Compute Stetson's variability indexes
    variability_indexes( jday, m, merr, i, number_of_measured_images_from_vast_summary_log, &I, &J, &K, &L, &J_clip, &L_clip, &J_time, &L_time, &I_sign_only, &N3, &excursions, &eta, &E_A, &SB);
    
    points_in_lightcurve=i;
    
    /// This thing has to be sorted in time!!!!
    //lag1_autocorrelation=gsl_stats_lag1_autocorrelation( m, 1, i);
    lag1_autocorrelation=lag1_autocorrelation_of_unsorted_lightcurve( jday, m, i);

    // Warning! After that point jday[i]-m[i]-merr[i] match is destroyed!
    // We keep only m[i]-merr[i] match by using gsl_sort2() instead of gsl_sort()
    gsl_sort2(m,1,merr,1,i);
    
    // Warning - lightcurve filtering below!!!!!
    /// Drop points
    if( i>STAT_MIN_NUMBER_OF_POINTS_FOR_NDROP && i>2*n_points_to_drop ){
     i-=n_points_to_drop;   
     for(j=0;j<i;j++){
      m[j]=m[j+n_points_to_drop];
      merr[j]=merr[j+n_points_to_drop];
     }
     i-=n_points_to_drop;
    }    

    //// Computations for the classical mag-sigma plot ////
    m_median=gsl_stats_median_from_sorted_data(m,1,i);
    sigma_series=gsl_stats_sd(m,1,i);
    // replacement of the old external filter
    if( 0==isnormal(m_median) )continue;
    if( fabs(m_median)>fabs(BRIGHTEST_STARS) )continue;
    if( fabs(m_median)<fabs(FAINTEST_STARS) )continue;
    if( sigma_series>MAX_MAG_ERROR )continue;
    //
    if( sigma_series==0.0 || 0==isnormal(sigma_series) )continue;
    //
    fprintf(legacy_data_file,"%10.6lf %8.6lf %9.3lf %9.3lf %s\n",m_median,sigma_series,x_ref,y_ref,ep->d_name);
    
    
    //// Computations for the new extended plot ////
    // compute weights
    //double old_m_mean;
    for(j=0;j<i;j++){w[j]=1.0/merr[j];}
    // Initial guess
    /////////// SHOULD WE ITERATE THIS????? ///////////
    m_mean=gsl_stats_wmean(w, 1, m, 1, i);
    // scale weights with a deviation from mean
    for(j=0;j<i;j++){
     d=(m[j]-m_mean)/merr[j];
     w[j]=w[j]/(1.0+pow(fabs(d)/a,b));
    }
    m_mean=gsl_stats_wmean(w, 1, m, 1, i);
    /////////// SHOULD WE ITERATE THIS????? ///////////
    modified_sigma_series=gsl_stats_wsd_m(w, 1, m, 1, i, m_mean);
    if( modified_sigma_series==0.0 || 0==isnormal(modified_sigma_series) )continue;
    //
    skewness=gsl_stats_wskew_m_sd(w, 1, m, 1, i, m_mean, modified_sigma_series);
    kurtosis=gsl_stats_wkurtosis_m_sd(w, 1, m, 1, i, m_mean, modified_sigma_series);

    // Scale modified_sigma by a number of points
    modified_sigma_series=modified_sigma_series*( (double)i/(double)number_of_measured_images_from_vast_summary_log );
    
    MAD_scaled_to_sigma=esimate_sigma_from_MAD_of_sorted_data( m, i);
    
    for(RoMS=0.0,j=0;j<i;j++){
     RoMS+=fabs( (m[j]-m_median)/merr[j] );
    }
    RoMS=RoMS/(double)(i-1);
    
    // Chi2
    double Chi2;
    for(j=0;j<i;j++){w[j]=1.0/(merr[j]*merr[j]);}
    m_mean=gsl_stats_wmean(w, 1, m, 1, i);
    for(Chi2=0.0,j=0;j<i;j++){
     Chi2+=(m[j]-m_mean)*(m[j]-m_mean)/(merr[j]*merr[j]);
    }
    Chi2=Chi2/(double)(i-1);
    
    // AGN-style variability index    
    //AGN_v=( (m[0]+merr[0])-(m[i-1]-merr[i-1]) )/( (m[0]+merr[0])+(m[i-1]-merr[i-1]) );
    // above was the old definition, I'll use my version instead
    double m_minus_sigma_max,m_plus_sigma_min;
    m_minus_sigma_max=m_plus_sigma_min=m[0];
    for(j=0;j<i;j++){
     if( m[j]-merr[j]>m_minus_sigma_max )m_minus_sigma_max=m[j]-merr[j];
     if( m[j]+merr[j]<m_plus_sigma_min )m_plus_sigma_min=m[j]+merr[j];
    }
    AGN_v= ( m_plus_sigma_min - m_minus_sigma_max ) / ( m_minus_sigma_max + m_plus_sigma_min );
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   1           2         3    4         5         6                      7       8     9  10 11 12   13                   14                   15                 16   17     18         19    20     21     22     23    24   25       26  27  28
    fprintf(extended_data_file,"%10.6lf %8.6lf %9.3lf %9.3lf %s  %6.4lf  %+10.4lf %+10.4lf  %+10.4lf %+10.4lf %+10.4lf %+10.4lf %5d %10.4lf %+10.4lf  %10.4lf %10.4lf %+10.4lf %10.4lf  %+10.4lf %+10.4lf %+10.4lf %+10.4lf  %+10.4lf  %+10.4lf  %+10.4lf %+10.4lf %+10.4lf\n",m_median,sigma_series,x_ref,y_ref,ep->d_name, modified_sigma_series,skewness,kurtosis,I, J, K, L, points_in_lightcurve,MAD_scaled_to_sigma,lag1_autocorrelation,RoMS,Chi2,I_sign_only,AGN_v,J_clip,L_clip,J_time,L_time,N3,excursions,eta,E_A,SB);
    
   }
  }
  (void) closedir (dp);
 }
 else
  perror("Couldn't open the directory");
  
 free(m);
 free(merr);
 free(w);
 free(jday);
 
 fclose(extended_data_file);
 fclose(legacy_data_file);

 sort_log_file_in_mag();
 
 return 0;
}
