#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "../vast_limits.h"
#include "../stetson_variability_indexes.h"
#include "../lightcurve_io.h"

int main(int argc, char **argv){

 double * JD;
 double * m;
 double * merr;
 
 int Nobs;
 int Nmax;
 
 double I,J,K,L,I_sign_only; // Stetson's variability indexes
 double J_clip,L_clip,J_time,L_time; // Modified Stetson's variability indexes
 double N3; // number of significant same-sign deviations
 double excursions;
 double eta,E_A,SB;
 
 double jd, mag, mag_err, x, y, app;
 char string[FILENAME_LENGTH];
 
 FILE *lc_file_descriptor;
 
 if( argc<2 ){
  fprintf(stderr,"Usage: %s test_lightcurve.dat\n",argv[0]);
  return 1;
 }

 lc_file_descriptor=fopen(argv[1],"r");
 if( lc_file_descriptor==NULL ){fprintf(stderr,"ERROR: cannot open lightcurve file %s\n",argv[1]);return 1;}
 Nobs=0;
 JD=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(double));
 m=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(double));
 merr=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(double));
 fprintf(stderr,"01\n");
 while( 0==read_lightcurve_point( lc_file_descriptor, &jd, &mag, &mag_err, &x, &y, &app, string) ){
  fprintf(stderr,"02\n");
  JD[Nobs]=jd;
  m[Nobs]=mag;
  merr[Nobs]=mag_err;
  fprintf(stderr,"03 %lf %lf %lf\n",jd,mag,mag_err);
  Nobs++;
 }
 fclose(lc_file_descriptor);
 Nmax=Nobs; // !!!
 
 fprintf(stderr,"04\n");

 variability_indexes( JD, m, merr, Nobs, Nmax, &I, &J, &K, &L, &J_clip, &L_clip, &J_time, &L_time, &I_sign_only, &N3, &excursions, &eta, &E_A, &SB);
 
 fprintf(stderr,"N = %d\nI = %lf\nJ = %lf %lf %lf\nK = %lf\nL = %lf %lf %lf\nI_sign_only = %lf\nN3 = %lf\nexcursions = %lf\neta = %lf\nE_A = %lf\nSB = %lf\n",Nobs,I,J,J_clip,J_time,K,L,L_clip,L_time,I_sign_only,N3,excursions, eta, E_A, SB);
 
 free(JD);
 free(m);
 free(merr);
 
 return 0;
}
