#!/usr/bin/env bash
#
# Convert all files in a specified directory to the VaST lightcurve 
# format *assuming* all files in the directory are ASCII lightcurves 
# in the "JD mag err" format.
#
###################################
SCRIPTNAME=`basename $0`
###################################
# Check input
if [ -z $1 ];then
 echo "The script $SCRIPTNAME will convert all files in a specified directory to the VaST lightcurve
format *assuming* all files in the directory are ASCII lightcurves in the \"JD mag err\" format.
 
Usage: $0 /path/to/directory/with/ascii/lightcurves"
 exit 1
fi
ASCII_LC_DIR="$1"
if [ ! -d "$ASCII_LC_DIR" ];then
 echo "ERROR: the specified path $ASCII_LC_DIR is no a directory"
 exit 1
fi
###################################
# Run the conversion for each file in the dir
for ASCII_LC_FILE in "$ASCII_LC_DIR"/* ;do
 if [ ! -f "$ASCII_LC_FILE" ];then
  echo "WARNING: $ASCII_LC_FILE is not a regular file and will not be converted!"
  continue
 fi
 file "$ASCII_LC_FILE" | grep "ASCII text"
 if [ $? -ne 0 ];then
  echo "WARNING: $ASCII_LC_FILE is not an ASCII text and will not be converted!"
  continue
 fi
 # Make-up an out*.dat name for the output file
 NEW_NAME=`basename "$ASCII_LC_FILE" .dat`
 NEW_NAME=`basename "$NEW_NAME" .txt`
 NEW_NAME=out_"$NEW_NAME".dat
 cat "$ASCII_LC_FILE" | awk '{print $1" "$2" "$3" 0.1 0.1 1.0 none"}' > "$NEW_NAME"
done
